
{% for response_task in objects %}
[panel://workbench_panel_{{ response_task.lowercase_name }}___response_task]
label = {{ response_task.name }}
description = {{ response_task.description }}
disabled = 0
tokens = {\
{% for token in response_task.inputs %}
{% if token == 'user' %}
    "user": {\
      "valuePrefix": "\"",\
      "valueSuffix": "\"",\
      "delimiter": " OR {{ token }}=",\
      "valueType": "primitive",\
      "value": "identity",\
      "default": "null"\
    }{% elif token == 'dest'%}
    "dest": {\
      "valuePrefix": "\"",\
      "valueSuffix": "\"",\
      "delimiter": " OR {{ token }}=",\
      "valueType": "primitive",\
      "value": "asset",\
      "default": "null"\
    }{% else %}
    "{{ token }}": {\
      "valuePrefix": "\"",\
      "valueSuffix": "\"",\
      "delimiter": " OR {{ token }}=",\
      "valueType": "primitive",\
      "value": "file",\
      "default": "null"\
    }{% endif %}{{ "," if not loop.last }}\
{% endfor %}
}\


{% endfor %}