---
title: {{ object['name'] }}
layout: tag
author_profile: false
taxonomy: {{ object['name'] }}
permalink: /detections/{{ object['name'] | lower | replace(' ', '_') }}/
sidebar:
  nav: "detections"
---
