---
title: "{{object.name}}"
last_modified_at: {{object.date}}
toc: true
toc_label: ""
tags:
  - {{ object.type }}
{%- for product in object.tags.product %}
  - {{ product }}
{%- endfor -%}
{%- for app in object.app_list %}
  - {{ app }}
{%- endfor %}
{%- for tag in object.tags.platform_tags %}
  - {{ tag }}
{%- endfor %}
---

[Try in Splunk SOAR](https://www.splunk.com/en_us/software/splunk-security-orchestration-and-automation.html){: .btn .btn--success}

#### Description

{{ object.description }}

- **Type**: {{ object.type }}
- **Product**: {{ object.tags.product|join(', ') }}
- **Apps**: {% for app in object.app_list %}[{{ app }}](https://splunkbase.splunk.com/apps/#/search/{{ app }}/product/soar){% if not loop.last %}, {% endif %}{%-endfor %}
- **Last Updated**: {{ object.date }}
- **Author**: {{object.author}}
- **ID**: {{ object.id }}

#### Associated Detections
{% if object.tags.detection_objects %}
{% for detection in object.tags.detection_objects -%}
* [{{ detection.name }}](/detection/{{ detection.lowercase_name }}/)
{% endfor %}
{% endif %}

#### How To Implement
{{ object.how_to_implement}}

#### Playbooks
![](https://raw.githubusercontent.com/splunk/security_content/develop/playbooks/{{ object.playbook | lower | replace(" ", "_")}}.png)

#### Required field
{% for field in object.tags.object_fields -%}
* {{ field }}
{% endfor %}

#### Reference
{% if object.references %}
{% for reference in object.references -%}
* [{{ reference }}]({{ reference }})
{% endfor %}
{% endif %}


[*source*](https://github.com/splunk/security_content/tree/develop/playbooks/{{ object.name | lower | replace (" ", "_") }}.yml) \| *version*: **{{object.version}}**
